####################################################################################################
# This R-script calcualtes and plots the fixation probability of a beneficial mutation in a        #
# changing environment.                                                                            #
#                                                                                                  #
# Mutants have a constant selective advantage s over the resident.                                 #
# The total population size increases or decreases logistically.                                   #
#                                                                                                  #
# Initial poluation size is given by kappa*pop_size, where pop_size is the carrying capacity.      #
#                                                                                                  #  
# The script plots the effective selection coefficient as a function of the parameter kappa.       #
# The plots show the effective selection coefficient calcualted from the simulations, and the      #
# correpsonding 99% confidence intervals.                                                          #
#                                                                                                  #
####################################################################################################


# population size
pop_size<-50000

# growth rate 
re<-exp(0.01)

# maximum number of generations 
generations<-50000

# number of fixation events
number_fix<-0

# number of replicates per parameter combination
reps<-5000

# selection paramters
s<-0.001

# number of parameter values for kappa
steps <- 4


kappavec <-mat.or.vec(1,steps)
fix_vec <- mat.or.vec(1,steps)

# values of kappa 
kappavec <- c(0.01,0.1,1,10)

for (jj in c(1:steps))   # loop that goes through all values of kappavec
{
  kappa <- kappavec[jj]   # pick kappa 
  number_fix<-0

  for (ii in c(1:reps))   # simulate replicates for one value of kappa
  { 

 
    muts <- 1                   # initial number of mutations = 1
    resi <- kappa*pop_size - muts     # initial number of residents
    tot_ind<-kappa*pop_size           # total number of individuals  


    for(g in c(0:generations))    
    {
      # selection coefficient in generation g
      sn <- s
      
      # calculate expected numer of offspring of a resident (according to beverton-holt equation)
      lambda <- re/( 1 + (tot_ind*(re-1)/pop_size))
    
      # calculate expected number of offspring of a mutant
      lambda_mut <-lambda*(1+sn)
    
      # mutant offspring
      muts <- sum(rpois(muts,lambda_mut))

      # resident offspring
      resi <- sum(rpois(resi,lambda))
     
      tot_ind<- muts + resi
      
      if (muts > 500)   # we stop if there are more than 500 mutants alive and count this as fixation 
      {
        number_fix <- number_fix+1
        break
      }
      
      
      if (muts==0)   # if no mutant leaves descendants we stop and count this as extinction
      {
    
        break
  
      }
    
    }
  
  }
  fix_vec[jj]<-number_fix/reps  # calculate fixation probability 
}

qv<-fix_vec   
w<-2.57* sqrt(qv *(1-qv)/reps) # calcualte 99% confidence interval

#plot confidence intervals
plotCI(kappavec[c(1:steps)],qv,uiw=w,col="black",ylim=c(0.0,0.065),xlab=expression(N[0]/K),ylab=expression(p),lwd=2,pch=20,gap=0,log="x")
